<?php

namespace WPIS;

include_once $_SERVER['DOCUMENT_ROOT'] . '/wp-config.php';
include_once $_SERVER['DOCUMENT_ROOT'] . '/wp-load.php';
include_once $_SERVER['DOCUMENT_ROOT'] . '/wp-includes/wp-db.php';
include_once $_SERVER['DOCUMENT_ROOT'] . '/wp-includes/pluggable.php';

class ApiPublic {

    const NAMESPACE = 'wpis';

    private $apiUrl = WPIS::API_URL . '/realestate/';
    private $postTypeEstateName;

    static public function init() {

        $class = new self();

		$class->wp_db = $GLOBALS['wpdb'];
		$class->wp_table = $class->wp_db->prefix . 'wpis_connexion';

		$class->headers[] = 'Cache-Control: no-cache';
		$class->headers[] = 'Accept: application/json';
		$class->headers[] = 'Content-Type: application/json';

        add_action( 'wp_ajax_nopriv_wpis_' . 'createContact', [$class, 'createContact'] );
        add_action( 'wp_ajax_wpis_ '. 'createContact', [$class, 'createContact'] );
        add_action( 'rest_api_init', [ $class, 'registerRoutes' ] );

        $class->postTypeEstateName =
            SettingsManager::get('enableImportCustomPostType') ?
            SettingsManager::get('customPostType') :
            EstatePostType::$postTypeName;

        return $class;
    }

    public function registerRoutes() {
        register_rest_route(
            self::NAMESPACE,
            '/' . 'estates',
            [
                [
                    'methods'  => 'GET',
                    'callback' => [ $this, 'getEstates' ],
                    // 'permission_callback' => [ $this, 'get_items_permission_check' ],
                    'args' => [
                        'page'     => [ 'required' => false ],
                        'limit'    => [ 'required' => false ],
                        'filters'  => [ 'required' => false ],
                    ]
                ]
            ]
        );
        register_rest_route(
            self::NAMESPACE,
            '/' . 'filters',
            [
                [
                    'methods'  => 'GET',
                    'callback' => [ $this, 'getFilters' ],
                ]
            ]
        );
    }

    public function getEstates( $request ) {

        $page = $request->get_param('page');
        $limit = $request->get_param('limit');
        $sort = $request->get_param('sort');
        $excludeProject = true;

        $filters = $request->get_param('filters') ?? false;
        $filters = json_decode($filters);

        $metaQuery = [];

        if( $excludeProject ){
            $metaQuery[] = [
                'relation' => 'OR',
                [
                    'key' => 'wpis_properties_hasChildren',
                    'value' => '1',
                    'compare' => '!=',
                ],
                [
                    'key' => 'wpis_properties_hasChildren',
                    'compare' => 'EXISTS'
                ]
            ];
        };

        if( isset($filters->purpose) && 'null' !== $filters->purpose ){
            $metaQuery[] = [
                'key'     => 'wpis_purpose_label',
                'value'   => $filters->purpose,
                'compare' => '=',
            ];
        }

        if( isset($filters->category) && 'null' !== $filters->category ){
            $metaQuery[] = [
                'key'     => 'wpis_category_label',
                'value'   => $filters->category,
                'compare' => '=',
            ];
        }

        if( $filters->cities && 'null' !== $filters->cities ){
            if( is_array($filters->cities) ){
                $meta = [];
                $meta['relation'] = 'OR';
                foreach( $filters->cities as $city ){
                    $meta[] = [
                        'key'     => 'wpis_address_city',
                        'value'   => $city,
                        'compare' => '=',
                    ];
                }
                $metaQuery[] = $meta;
            }
            else {
                $metaQuery[] = [
                    'key'     => 'wpis_address_city',
                    'value'   => $filters->cities,
                    'compare' => '=',
                ];
            }
        }

        if( $filters->priceRange ){
            $filters->priceRange[1] = $filters->priceRange[1] === -1 ? 1e10 : $filters->priceRange[1];
            $metaQuery[] = [
                'key'     => 'wpis_finance_price',
                'value'   => $filters->priceRange,
                'compare' => 'BETWEEN',
                'type'    => 'SIGNED'
            ];
        }

        if( $filters->bedroomRange && $filters->bedroomRange[0] !== 0 ){
            $metaQuery[] = [
                'key'     => 'wpis_configuration_bedrooms',
                'value'   => $filters->bedroomRange,
                'compare' => 'BETWEEN',
                'type'    => 'SIGNED'
            ];
        }

        if( $filters->bathroomRange && $filters->bathroomRange[0] !== 0 ){
            $metaQuery[] = [
                'key'     => 'wpis_configuration_bathrooms',
                'value'   => $filters->bathroomRange,
                'compare' => 'BETWEEN',
                'type'    => 'SIGNED'
            ];
        }

        if( $filters->areaRange && $filters->areaRange[0] !== 0 ){
            $metaQuery[] = [
                'key'     => 'wpis_areas_total',
                'value'   => $filters->areaRange,
                'compare' => 'BETWEEN',
                'type'    => 'SIGNED'
            ];
        }

        if( $filters->hideSoldEstates ){
            $metaQuery[] = [
                'key'     => 'wpis_status_id',
                'value'   => 3,
                'compare' => '!=',
                'type'    => 'SIGNED'
            ];
        }

        $args = [
            'paged'          => $page,
            'posts_per_page' => $limit,
            'post_type'      => $this->postTypeEstateName,
            'meta_query'     => $metaQuery
        ];

        if( $sort ){
            if( !in_array( $sort, ['price-asc', 'price-desc', 'date-asc', 'date-desc'] ) ){
                $sort = 'ASC';
            }
            $sort = strtoupper($sort);
            $sortData = explode('-', $sort);
            $type = $sortData[0];
            $sort = $sortData[1];

            if( $type === 'PRICE' ) {
                $args['meta_key'] = 'wpis_finance_price';
                $args['orderby']  = 'meta_value_num';
                $args['order']    = $sort;
            }
            if( $type === 'DATE' ) {
                $args['orderby']  = 'date';
                $args['order']    = $sort;
            }
        }

        $query = new \WP_Query($args);
        $estates = [];

        foreach( $query->posts as $post ){
            $post->featured_image = get_the_post_thumbnail_url($post->ID, 'medium_large');
            $post->post_metas = [];
            foreach( PostTypesManager::$postTypeMetas as $group){
                foreach($group['fields'] as $property){
                    $meta = 'wpis_' . str_replace('.', '_', $property['field']);
                    $post->post_metas[$meta] = get_post_meta($post->ID, $meta, true);
                }
            }
            $estates[] = $post;
        }

        return rest_ensure_response( [
            'total' => $query->found_posts,
            'limit' => $limit,
            'page'  => $page,
            'data'  => $estates
        ] );
    }

    public function getFilters( $request ) {

        $filtersConfiguration = [
            'enabledfilters' => [
                'purpose_dropdown',
                'category_dropdown',
                'city_dropdown',
                'price_dropdown',
                // 'price_slider',
                'bedroom_slider',
                'bathroom_slider',
                'area_slider',
            ],
            'values' => []
        ];

        $estatesSellIDs = $this->getPostsByPurpose('À vendre', $this->postTypeEstateName);
        $estatesRentIDs = $this->getPostsByPurpose('À louer', $this->postTypeEstateName);

        $filtersConfiguration['values']['categories'] = $this->getMetaValues('wpis_category_label', $this->postTypeEstateName );
        $filtersConfiguration['values']['cities'] = $this->getMetaValues('wpis_address_city', $this->postTypeEstateName );
        $filtersConfiguration['values']['purposes'] = $this->getMetaValues('wpis_purpose_label', $this->postTypeEstateName );
        $filtersConfiguration['values']['areas'] = $this->getMetaMinMax('wpis_areas_total', $this->postTypeEstateName);
        $filtersConfiguration['values']['bedrooms'] = $this->getMetaMinMax('wpis_configuration_bedrooms', $this->postTypeEstateName);
        $filtersConfiguration['values']['bathrooms'] = $this->getMetaMinMax('wpis_configuration_bathrooms', $this->postTypeEstateName);
        $filtersConfiguration['values']['prices'] = [
            'selling' => $this->getMetaMinMax('wpis_finance_price', $this->postTypeEstateName, 'publish', $estatesSellIDs),
            'renting' => $this->getMetaMinMax('wpis_finance_price', $this->postTypeEstateName, 'publish', $estatesRentIDs)
        ];

        return rest_ensure_response( $filtersConfiguration );
    }

    private function getMetaValues( $key = '', $type = 'post', $status = 'publish' ) {

        global $wpdb;

        if( empty( $key ) )
            return [];

        $values = $wpdb->get_col( $wpdb->prepare( "
            SELECT DISTINCT pm.meta_value FROM {$wpdb->postmeta} pm
            LEFT JOIN {$wpdb->posts} p ON p.ID = pm.post_id
            WHERE pm.meta_key = %s
            AND p.post_status = %s
            AND p.post_type = %s
            ORDER BY pm.meta_value ASC
        ", $key, $status, $type ) );

        $associativeArray = [];

        foreach($values as $value){
            $associativeArray[] = [
                'key' => $value,
                'value' => $value
            ];
        }

        return $values;
    }

    private function getMetaMinMax( $key = '', $type = 'post', $status = 'publish', $posts = [] ) {

        global $wpdb;

        $postsCondition = '';
        if( !empty($posts) ){
            $postsCondition = 'AND p.id IN (' . implode(', ', $posts) . ')';
        }

        $range = $wpdb->get_row( $wpdb->prepare( "

        SELECT MIN(CAST(meta_value AS SIGNED)) AS min, MAX(CAST(meta_value AS SIGNED)) AS max FROM (
            SELECT pm.meta_value FROM {$wpdb->postmeta} pm
            LEFT JOIN {$wpdb->posts} p ON p.ID = pm.post_id
            WHERE pm.meta_key = %s
            AND p.post_status = %s
            AND p.post_type = %s"
            . $postsCondition .
        ") tmp
        ", $key, $status, $type ) );

        return [0, (int) $range->max];
    }

    private function getPostsByPurpose( $purpose = 'À vendre', $type = 'post', $status = 'publish' ){

        global $wpdb;

        $values = $wpdb->get_col( $wpdb->prepare( "
            SELECT p.ID FROM {$wpdb->posts} p
            LEFT JOIN {$wpdb->postmeta} pm ON p.ID = pm.post_id
            WHERE pm.meta_key = 'wpis_purpose_label'
            AND pm.meta_value = %s
            AND p.post_status = %s
            AND p.post_type = %s
        ", $purpose, $status, $type ) );

        return $values;
    }

    // OLD STUFF
    public function createContact(){

        $payload = json_decode(file_get_contents('php://input'));
        $acceptedSource = ['searchForm', 'contactForm', 'estateForm'];

        Logger::log('wpis-contact', '[NEW CONTACT]' );
        Logger::log('wpis-contact', '> rawInput: ' . json_encode($payload) );

        if( !isset($payload->source) || !in_array($payload->source, $acceptedSource) ){
            Logger::log('wpis-contact', '> error: ' . 'Contact source is not valid' );
            return $this->response(422, ['success' => false, 'data' => 'Contact source is not valid' ]);
        }

        Logger::log('wpis-contact', '> source: ' . $payload->source );

        $searchZips = [];
        foreach( [1, 2, 3, 4, 5, 6, 7, 8, 9 , 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25] as $zipKey ){
            if( isset($payload->{'searchZipCode' . $zipKey}) && $payload->{'searchZipCode' . $zipKey}){
                $searchZips[] = $payload->{'searchZipCode' . $zipKey};
            }
        }

        $input = [
            'source'    => $payload->source,
            'phone'     => $payload->phone,
            'firstname' => $payload->firstname,
            'lastname'  => $payload->lastname,
            'email'     => $payload->email,

            'agreement' => isset($payload->agreement) && $payload->agreement,

            'addressNumber' =>  $payload->addressNumber,
            'addressZip'    =>  $payload->addressZip,
            'addressStreet' =>  $payload->addressStreet,
            'addressCity'   =>  $payload->addressCity,
            'addressBox'    =>  $payload->addressBox,

            'message' => $payload->comment || $payload->message
        ];

        if( $payload->officeId ){
            $input['officeId'] = $payload->officeId;
        }

        if( $payload->source === 'searchForm' ){
            $input['search'] = [
                'purpose'    => $payload->searchPurpose,
                'category'   => $payload->searchCategory,
                'priceRange' => [$payload->searchPriceMin, $payload->searchPriceMax],
                'rooms'      => $payload->searchRooms,
                'zips'       => $searchZips
            ];
        }

        if( $payload->source === 'estateForm' ){
            $input['estateInterestId'] = $payload->estateId;
            $input['message'] = 'ID: ' . $payload->estateId . ' | MESSAGE: '. $payload->comment;
        }

        Logger::log('wpis-contact', '> formatedInput: ' . json_encode($input) );

        $response = $this->request('POST', 'contact', $input, $this->headers);

        Logger::log('wpis-contact', '> response : ' . json_encode($response) . "\n" );

        return $this->response(200, ['success' => true, 'response' => $response]);
    }

    public function request(string $method, string $endpoint, array $payload = []){

        $payload['wpis_version'] = get_plugin_data(__DIR__ . '/../wp-immo-sync.php')['Version'];
        $payload['wp_version']   = get_bloginfo('version');;
        $payload['referer']      = $_SERVER['HTTP_HOST'];

        $this->checkAuth();

        $method = strtoupper($method);

        $params = http_build_query($payload);
        $curl = curl_init();

        $options = [
            CURLOPT_URL => $this->apiUrl . $endpoint . '?' . $params,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 30,
            CURLOPT_CUSTOMREQUEST => $method,
            CURLOPT_HTTPHEADER => $this->headers,
        ];

        if( $method === 'POST' ){
            $options[CURLOPT_POST] = true;
            $options[CURLOPT_POSTFIELDS] = http_build_query($payload);
        }

        curl_setopt_array($curl, $options);

        $response = curl_exec($curl);
        $err = curl_error($curl);
        $httpcode = curl_getinfo($curl, CURLINFO_HTTP_CODE);

        curl_close($curl);

        if ($err)
            return (object) ['error' => true, 'message' => "cURL Error : " . $err];

        return (object) ['code' => $httpcode, 'data' => $response];
    }

    private function response( $code, $data = false ) {
        http_response_code($code);
        header('Content-Type: application/json');
        if( $data ){
            echo json_encode($data);
        }
        exit;
    }

    private function checkAuth(){

		$connectionInfo = $this->wp_db->get_results("SELECT * FROM $this->wp_table LIMIT 1");

		if( empty($connectionInfo) ){
			$this->errors = 'Wordpress is not connected to WP Immo Connect platform.';
			return $this;
		}

		$protocol = (!empty($_SERVER['HTTPS']) && $_SERVER['HTTPS'] !== 'off' || $_SERVER['SERVER_PORT'] == 443) ? "https://" : "http://";

		$this->headers[] = 'Access-Control-Allow-Origin: ' . $protocol . $_SERVER['HTTP_HOST'];
		$this->headers[] = 'Authorization: Bearer ' . $connectionInfo[0]->token;

		return $this;
	}

}